##----------------------------------------------------------------------------##
##=Purpose: Exercise 5.2 of Beyond Classic Designs & Analysis for Health      ##
##          Data in Singapore (27 June to 8 July 2022)                        ##
##
##=Data required: 
##  - tiny_cohort.dta
##  - tinycohort2.dta
##  - skeleton_cohort.dta 
##  - 1-2 NCC sample.dta
##
##=R packages/libraries required:
##  - dplyr: 
##    * for processing data
##  - haven:
##    * read_dta: To read in recent versions of STATA dta files.
##  - SamplingDesignTools:
##    * compute_km_weights: for computing KM weights
##      link: https://github.com/nyilin/SamplingDesignTools
##  - survey:
##    * twophase: Specify the two stage design information
##    * svyglm: Perform glm analysis with sampling weights
##  - survival:
##    * clogit: To perform conditional logistic regression.
##
##=Created by:
##  - Francesca Graziano 
##  - Zilu Feng
##  - Zihan 
##  - Chuen Seng Tan
##  - Marie Reilly
##
##=Changes (if any):
##  -DDMMYYYY: Describe
##
##----------------------------------------------------------------------------##

##-Set the working directory to the data folder where the datasets are saved:
setwd("D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data")

##-R libraries to install:
#install.packages(c("dplyr","haven","SamplingDesignTools","survey","survival"))
##-Load the R libraries
library(dplyr)
library(haven)
library(SamplingDesignTools)
library(survey)
library(survival)

################################################################################
##                               Question 1 (i)     
## 
##-(i) In these exercises, you will calculate the Kaplan-Meier weights by hand for 
##     a simple example of a nested case-control sample. You will also run 
##     software commands to compute these weights for different kinds of data that 
##     may be available in a research study. The first command "cohort_km_weights" 
##     is used when individual data is available for the outcome and the sampling 
##     strata variables for all cohort members from which the nested case-control 
##     sample has been taken (or is to be taken): this would be the situation where 
##     a researcher selects a nested case-control sample from an electronic cohort 
##     that they have available. The second command "sampled_km_weights" is used 
##     when the researcher has a nested case-control sample but the population 
##     counts within the sampling strata are only available from an external 
##     source, for example from national statistics offices. 
################################################################################


################################################################################
##                               Question 1 (ii)     
##
##-(ii)	Compare your weights in (i) to the weights obtained from the 
##      appropriate software command (see the hints at the end of this document).
################################################################################

#Using the cohort data with chosen column indicating 1:
#1- import dataset 
tiny1 <- haven::read_dta("tiny_cohort.dta")

#2- generate the variable for the sample_stat argument which is a numeric variable:
#   0 for non-sampled controls, 1 for sampled (and kept) controls, and 
#   integers >=2 for events.
tiny1 <- tiny1 %>% dplyr::mutate(sample_stat = event + chosen)

#3- generate the km weight using function from SamplingDesignTools
t1.km <- compute_km_weights(cohort = tiny1, 
                            t_name = "t", y_name = "event",
                            sample_stat = tiny1$sample_stat, 
                            n_per_case = 1)


################################################################################
##                               Question 1 (ii)     
##
##-(iii) Repeat steps (i) -(ii) for "tinycohort2.dta" which is similar to 
##       "tiny_cohort" but has two shared event times (see subsequent page). .
################################################################################

#1- import dataset 
tiny2 <- haven::read_dta("tiny_cohort2.dta")

#2- generate the variable for the sample_stat argument which is a numeric variable:
#   0 for non-sampled controls, 1 for sampled (and kept) controls, 
##  and integers >=2 for events.
tiny2 <- tiny2 %>% dplyr::mutate(sample_stat = event + chosen)

#3- generate the km weight using function from SamplingDesignTools
t2.km <- compute_km_weights(cohort = tiny2, t_name = "t", y_name = "event",
                            sample_stat = tiny2$sample_stat, 
                            n_per_case = 1)


################################################################################
##                               Question 2 (i)     
##
##-(i) Use the tiny_cohort data set from Q1 to create a file with the risk sets 
##     and number of failures using an appropriate survival analysis command 
##     and save this as a temporary data set.
################################################################################

#1- generate the risk table from function in SamplingDesignTools package
risk_table1 <- compute_risk_table(cohort = tiny1, t_name = "t", y_name = "event")


################################################################################
##                               Question 2 (ii)     
##
##-(ii) Open "tiny_cohort" again and select just the "chosen" NCC individuals 
##      as your case-control study data.
################################################################################

#1- generate the ncc data
tiny1.ncc <- tiny1 %>%
  dplyr::filter(chosen==1)



################################################################################
##                               Question 2 (iii)     
##
##-(iii) Use the appropriate software command and the external risk set 
##       information you saved in (i) to obtain the weights for the individuals 
##       in this NCC sample.
################################################################################


#1- Need to generate the NCC dataset where the the rows the controls  and cases 
#   from the same match set
#1a Identify (case id, control id) pairs (6 pairs in total): 
#   (3, 4), (5,7), (6,9), (9,11), (10,13), (12,14)
id1.pair.mat<-rbind(c(3, 4),c(5,7),c(6,9),c(9,11),c(10,13),c(12,14))
#1b convert the pair into a vector:
id1.pair.vec<-c(t(id1.pair.mat))
#1c create a case status (=1) indicator based on the pair-vector:
id1.pair.case.status<-rep(c(1,0),times=nrow(id1.pair.mat))
#1d create a vector of the event time of case for each match set based on pair-vector:
ii1<-match(id1.pair.mat[,1],tiny1.ncc$id)
id1.pair.case.time<-rep(tiny1.ncc$t[ii1],each=2)
#1d create a vector of the event/censor time of the case/control based on pair-vector:
jj1<-match(id1.pair.vec,tiny1.ncc$id)
id1.pair.cc.time<-tiny1.ncc$t[jj1]

tiny1.ncc.dat<-data.frame(set = rep(1:nrow(id1.pair.mat),each=2),
                          id = id1.pair.vec,
                          case.status = id1.pair.case.status,
                          t.event = id1.pair.case.time,
                          t = id1.pair.cc.time)

tiny1.km2 <- tiny1.ncc.dat %>% select(-set) %>% 
  compute_km_weights(ncc = ., 
                     risk_table_manual = risk_table1, 
                     id_name = "id", 
                     t_name = "t", y_name = "case.status", t_match_name = "t.event",
                     n_per_case = 1)


################################################################################
##                               Question 2 (iv)     
##
##-(iv) Compare the results from (iii) to what you obtained from the 
##      alternative weighting using the skeleton of the cohort in Q1.
################################################################################

#They are in agreement


################################################################################
##                               Question 2 (v)     
##
##-(v) Repeat the steps for the data with tied event times ("tiny_cohort2"). 
##     NOTE: the riskset information must only have one observation for each 
##     time point. 
################################################################################

##===Second tiny cohort
#1- generate the risk table from function in SamplingDesignTools package
risk_table2 <- compute_risk_table(cohort = tiny2, t_name = "t", y_name = "event")


#3- generate the ncc data
tiny2.ncc <- tiny2 %>%
  dplyr::filter(chosen==1)


#4- Need to generate the NCC dataset where the the rows the controls  and cases 
#   from the same match set
#4a Identify (case id, control id) pairs (6 pairs in total): 
#   (3, 4), (5,7), (6,9), (9,11), (10,13), (12,14)
#   Note that (5, 7) and (6, 9) is one specific configuration of sets given these
#   four observations are matched in the same event time.
#   Similarly, (9, 11) and (10, 13) is one specific configuration of sets given these
#   four observations are matched in the same event time.
#   For the R package used, any configuration would give the same KM weights
id2.pair.mat<-rbind(c(3, 4),c(5,7),c(6,9),c(9,11),c(10,13),c(12,14))
#4b convert the pair into a vector:
id2.pair.vec<-c(t(id2.pair.mat))
#4c create a case status (=1) indicator based on the pair-vector:
id2.pair.case.status<-rep(c(1,0),times=nrow(id2.pair.mat))
#4d create a vector of the event time of case for each match set based on pair-vector:
ii2<-match(id2.pair.mat[,1],tiny2.ncc$id)
id2.pair.case.time<-rep(tiny2.ncc$t[ii2],each=2)
#4d create a vector of the event/censor time of the case/control based on pair-vector:
jj2<-match(id2.pair.vec,tiny2.ncc$id)
id2.pair.cc.time<-tiny2.ncc$t[jj2]

tiny2.ncc.dat<-data.frame(set = rep(1:nrow(id2.pair.mat),each=2),
                          id = id2.pair.vec,
                          case.status = id2.pair.case.status,
                          t.event = id2.pair.case.time,
                          t = id2.pair.cc.time)

tiny2.km2 <- tiny2.ncc.dat %>% select(-set) %>% 
  compute_km_weights(ncc = ., 
                     risk_table_manual = risk_table2, 
                     id_name = "id", 
                     t_name = "t", y_name = "case.status", t_match_name = "t.event",
                     n_per_case = 1)


################################################################################
##                               Question 3     
##
##-3. The data set skeleton_cohort.dta contains "skeleton" information for the 
##    cohort of 50,000 individuals that you analysed in Exercise 3.2: just the 
##    ID, the time (t, in years) from enrolment to coronary heart disease (CHD), 
##    and an event indicator 1=CHD, 0=censored. The data set 
##    "1-2 NCC sample.dta" is a 1:2 nested case-control sample drawn from 
##    the same cohort with all variables recorded for the selected individuals: 
##    - id (unique identifier)
##    - age (Age, in years)
##    - gender (1=Male, 0=Female)
##    - Treat (antihypertensive treatment status 1=Yes, 0=No) 
##    - Smoke (smoking status 1=Yes, 0=No)
##    - Chol (cholesterol in mg/dL)
##    - HDL (high-density lipoprotein in mg/dL)
##    - SBP (systolic blood pressure in mmHg)
################################################################################



################################################################################
##                               Question 3 (i)     
##
##-(i) Generate the age category variable agecat using categories 
##      <=49, 50-59, 60-69, 70-79, 80+ and verify that the OR from 
##      conditional logistic regression of the nested case-control data 
##      provides similar estimates to the HRs below from the full cohort 
##      which were obtained in Exercise 3.2 
##      [see exercise]
################################################################################

#1- read in the data
ncc.dat <- haven::read_dta("1-2 NCC sample.dta")
#rename the columns for _case and _set
colnames(ncc.dat)[colnames(ncc.dat)=="_case"]<-"case"
colnames(ncc.dat)[colnames(ncc.dat)=="_set"]<-"set"

#2- categorized age
ncc.dat$agecat<-cut(ncc.dat$Age,breaks=c(0,49,59,69,79,100))
table(ncc.dat$agecat)

#3- mean center Chol, HDL, SBP
ncc.dat <- ncc.dat %>% 
  mutate(Chol = Chol - mean(Chol), 
         HDL = HDL - mean(HDL), 
         SBP = SBP - mean(SBP))

#4- fit conditional logistic
clogit.ncc.dat<-clogit(case~Chol+HDL+SBP+Gender+Treat+Smoke+factor(agecat)+strata(set),
                       data=ncc.dat)
#key results
round(cbind(exp(cbind(OR=clogit.ncc.dat$coefficients,
                      confint(clogit.ncc.dat))),
            p.value=summary(clogit.ncc.dat)$coefficients[,"Pr(>|z|)"]),3)


################################################################################
##                               Question 3 (ii)     
##
##-(ii)	Use the nested case-control data to create a data set of unique 
##      individuals, by breaking the matching and keeping just one record
##      per person: the case record for any case who was previously a control, 
##      and just one record for any individual sampled more than once as a 
##      control (ensure that the data contains the entry time, and the event 
##      or censoring time, of all individuals)
################################################################################

#1- Identify unique individuals
ncc.nodup<-unique(ncc.dat[,c("id","t","event")])


################################################################################
##                               Question 3 (iii)     
##
##-(iii) From the skeleton of the cohort, find the weights for all individuals 
##       and merge these to the data from (ii) using the unique ID 
##       (Note: the weight should be 1 for cases as all cases were selected).
################################################################################

#1- read in the skeleton of the cohort
skeleton <- haven::read_dta("skeleton_cohort.dta")

#2- generate the sample_stat argument for compute_km_weights with event and chosen indicator
skeleton$sample_stat <- skeleton$event + as.numeric(skeleton$id%in%ncc.nodup$id)

#3- to determine number of controls per case from the ncc data
table(table(ncc.dat$set))
table(ncc.dat$case)
3840/1920

#4- generate the km weight using function from SamplingDesignTools
km <- compute_km_weights(cohort = skeleton, 
                         t_name = "t", y_name = "event",
                         sample_stat = skeleton$sample_stat, 
                         n_per_case = 2)


################################################################################
##                               Question 3 (iv)     
##
##-(iv) Sketch the weighted Kaplan-Meier curve and compare to the Kaplan-Meier 
##      curve from the skeleton cohort.
################################################################################

#1- merge the unique list of individuals and KM-weights
ncc.dat.wts<-merge(ncc.nodup,
                   unique(km[,c("t","km_weight")]),
                   by="t",all.x=TRUE)

#2- set-up the weights in survey
ncc.svy <- survey::svydesign(id=~1, weights=~km_weight, strata=NULL, 
                             data=ncc.dat.wts)

#3- sketch the weighted Kaplan-Meier curve
plot(survey::svykm(Surv(t,event)~1, design=ncc.svy),
     ylim=c(0.92,1))

#4- sketch the Kaplan-Meier curve from the skeleton cohort
par(new=TRUE)
plot(survival::survfit(Surv(time=t, event=event) ~ 1, data = skeleton),
     ylim=c(0.92,1), col="red")
par(new=FALSE)


################################################################################
##                               Question 3 (v)     
##
##-(v) Conduct a weighted Cox regression of the nested case-control data and 
##     verify that the estimates are close to the estimates from the full 
##     cohort shown in (i) above.
################################################################################

#1- merge the unique list of individuals and KM-weights
ncc.dat.cox.wts<-merge(ncc.dat.wts,
                       unique(ncc.dat[,c("id","Chol","HDL","SBP","Gender","Treat","Smoke","agecat")]),
                       by="id",all.x=TRUE)

#2- set-up the weights in survey
ncc.cox.svy <- survey::svydesign(id=~1, weights=~km_weight, strata=NULL, 
                             data=ncc.dat.cox.wts)

#3- fit weighted Cox regression
cox.wt<-svycoxph(Surv(time=t, event=event) ~ Chol+HDL+SBP+Gender+Treat+Smoke+agecat, 
                 design=ncc.cox.svy)
#key results
round(cbind(exp(cbind(HR=cox.wt$coefficients,
                      confint(cox.wt))),
            p.value=summary(cox.wt)$coefficients[,"Pr(>|z|)"]),3)
