//----------------------------------------------------------------------------//
//=Purpose: Exercise 5.2 of Beyond Classic Designs & Analysis for Health      //
//          Data in Singapore (27 June to 8 July 2022)                        //
//
//=Data required: 
//  - tiny_cohort.dta
//  - tinycohort2.dta
//  - skeleton_cohort.dta 
//  - 1-2 NCC sample.dta
//
//=STATA packages/libraries required:

//
//=Created by:
//  - Zihan 
//  - Francesca Graziano 
//  - Zilu Feng
//  - Chuen Seng Tan
//  - Marie Reilly
//
//=Changes (if any):
//  -DDMMYYYY: Describe
//
//----------------------------------------------------------------------------//

//-Set the working directory to the data folder where the datasets are saved:
cd "D:/nbox/Education/Marie_short_course/Singapore course 2022/20220602/exercises/data"


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (i)     
// 
//-(i) In these exercises, you will calculate the Kaplan-Meier weights by hand for 
//     a simple example of a nested case-control sample. You will also run 
//     software commands to compute these weights for different kinds of data that 
//     may be available in a research study. The first command "cohort_km_weights" 
//     is used when individual data is available for the outcome and the sampling 
//     strata variables for all cohort members from which the nested case-control 
//     sample has been taken (or is to be taken): this would be the situation where 
//     a researcher selects a nested case-control sample from an electronic cohort 
//     that they have available. The second command "sampled_km_weights" is used 
//     when the researcher has a nested case-control sample but the population 
//     counts within the sampling strata are only available from an external 
//     source, for example from national statistics offices. 
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (ii)     
//
//-(ii)	Compare your weights in (i) to the weights obtained from the 
//      appropriate software command (see the hints at the end of this document).
////////////////////////////////////////////////////////////////////////////////

//load the data:
use tiny_cohort.dta, clear
cohort_km_weights, controls(1) id(id) time(t) failure(event)	

////////////////////////////////////////////////////////////////////////////////
//                               Question 1 (iii)     
//
//-(iii) Repeat steps (i) -(ii) for "tinycohort2.dta" which is similar to 
//       "tiny_cohort" but has two shared event times (see subsequent page). .
////////////////////////////////////////////////////////////////////////////////

//load the data:
use tiny_cohort2.dta, clear
cohort_km_weights, controls(1) id(id) time(t) failure(event)


////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (i)     
//
//-(i) Use the tiny_cohort data set from Q1 to create a file with the risk sets 
//     and number of failures using an appropriate survival analysis command 
//     and save this as a temporary data set.
////////////////////////////////////////////////////////////////////////////////


//load the data and declare survival data:
use tiny_cohort.dta, clear
stset t, failure(event=1)

//the number at risk just before time t_j (generate riskset at each event time):
sts generate riskset=n

//the number failing at time t_j (generate number of failures at each event time):
sts generate nfailure=d

//save the dataset:	
save riskset, replace

	
////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (ii)     
//
//-(ii) Open “tiny_cohort” again and select just the “chosen” NCC individuals 
//      as your case-control study data.
////////////////////////////////////////////////////////////////////////////////

use tiny_cohort.dta, clear
keep if chosen==1


////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (iii)     
//
//-(iii) Use the appropriate software command and the external risk set 
//       information you saved in (i) to obtain the weights for the individuals 
//       in this NCC sample.
////////////////////////////////////////////////////////////////////////////////

sampled_km_weights using riskset, controls(1) id(id) time(t) failure(event) riskset(riskset) numfail(nfailure)


////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (iv)     
//
//-(iv) Compare the results from (iii) to what you obtained from the 
//      alternative weighting using the skeleton of the cohort in Q1.
////////////////////////////////////////////////////////////////////////////////

//They are the same.



////////////////////////////////////////////////////////////////////////////////
//                               Question 2 (v)     
//
//-(v) Repeat the steps for the data with tied event times (“tiny_cohort2”). 
//     NOTE: the riskset information must only have one observation for each 
//     time point. 
////////////////////////////////////////////////////////////////////////////////

//load the data and declare survival data:
use tiny_cohort2.dta, clear
stset t, failure(event=1)

//the number at risk just before time t_j (generate riskset at each event time):
sts generate riskset=n

//the number failing at time t_j (generate number of failures at each event time):
sts generate nfailure=d

//keep only one record of riskset information for each time point:
duplicates drop t, force

save riskset1, replace

//select just the “chosen” NCC individuals:
use tiny_cohort2.dta, clear
keep if chosen==1

//compute weights:
sampled_km_weights using riskset1, controls(1) id(id) time(t) failure(event) riskset(riskset) numfail(nfailure)


////////////////////////////////////////////////////////////////////////////////
//                               Question 3     
//
//-3. The data set skeleton_cohort.dta contains “skeleton” information for the 
//    cohort of 50,000 individuals that you analysed in Exercise 3.2: just the 
//    ID, the time (t, in years) from enrolment to coronary heart disease (CHD), 
//    and an event indicator 1=CHD, 0=censored. The data set 
//    “1-2 NCC sample.dta” is a 1:2 nested case-control sample drawn from 
//    the same cohort with all variables recorded for the selected individuals: 
//    - id (unique identifier)
//    - age (Age, in years)
//    - gender (1=Male, 0=Female)
//    - Treat (antihypertensive treatment status 1=Yes, 0=No) 
//    - Smoke (smoking status 1=Yes, 0=No)
//    - Chol (cholesterol in mg/dL)
//    - HDL (high-density lipoprotein in mg/dL)
//    - SBP (systolic blood pressure in mmHg)
////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////
//                               Question 3 (i)     
//
//-(i) Generate the age category variable agecat using categories 
//      <=49, 50-59, 60-69, 70-79, 80+ and verify that the OR from 
//      conditional logistic regression of the nested case-control data 
//      provides similar estimates to the HRs below from the full cohort 
//      which were obtained in Exercise 3.2 
//
//─────────────┬────────────────────────────────────────────────────────────────
//_t │ Haz. Ratio   Std. Err.      z    P>|z|     [95% Conf. Interval]
//─────────────┼────────────────────────────────────────────────────────────────
//centered C~l │   1.006221   .0007014     8.90   0.000     1.004847    1.007596
//centered_HDL │   .9756035    .001985   -12.14   0.000     .9717207    .9795018
//centered_SBP │   1.013027   .0010992    11.93   0.000     1.010875    1.015183
//Gender       │   1.667794   .0836694    10.20   0.000      1.51161    1.840116
//Treat        │   1.301126   .0680688     5.03   0.000     1.174326    1.441618
//Smoke        │   1.729781   .0893758    10.61   0.000     1.563185    1.914131
//agecat       │
//50-59        │   1.253414   .2442627     1.16   0.246     .8554883    1.836432
//60-69        │   2.129328   .4035375     3.99   0.000     1.468683    3.087146
//70-79        │   3.395816   .6522993     6.36   0.000     2.330439     4.94824
//80-97        │   4.509782   .9920104     6.85   0.000     2.930339    6.940538
//─────────────┴────────────────────────────────────────────────────────────────
////////////////////////////////////////////////////////////////////////////////

//Load the data
use "1-2 NCC sample.dta", clear
	
//create age category, such as 28-49; 50-59......:
egen agecat=cut(Age), at(28,50,60,70,80,97)
		
//label agecat:
label variable agecat "age_category"
label define agecat1 28 "28-49" 50 "50-59" 60 "60-69" 70 "70-79" 80 "80-97"
label values agecat agecat1  
codebook agecat

//center cholesterol at its mean:
summarize Chol, meanonly
gen centered_Chol = Chol - r(mean)

//center HDL at its mean:
summarize HDL, meanonly
gen centered_HDL = HDL - r(mean)

//center SBP at its mean:
summarize SBP, meanonly
gen centered_SBP = SBP - r(mean)

//save the dataset:
save NCC, replace

//OR from conditonal logistic model:
clogit _case centered_Chol centered_HDL centered_SBP Gender Treat Smoke i.agecat, group(_set) or

	
//we can see that the OR from conditional logistic regression of the nested case-control data provides similar estimates to the HRs below from the full cohort which were obtained in Exercise 3.2.

////////////////////////////////////////////////////////////////////////////////
//                               Question 3 (ii)     
//
//-(ii)	Use the nested case-control data to create a data set of unique 
//      individuals, by breaking the matching and keeping just one record
//      per person: the case record for any case who was previously a control, 
//      and just one record for any individual sampled more than once as a 
//      control (ensure that the data contains the entry time, and the event 
//      or censoring time, of all individuals)
////////////////////////////////////////////////////////////////////////////////

use NCC, clear

duplicates report id
duplicates tag id, gen(dup_id) 
gen drop_prev_case=0
replace drop_prev_case=1 if dup_id==1 & _case==0 & event==1
replace drop_prev_case=1 if dup_id==2 & _case==0 & event==1
drop if drop_prev_case==1
duplicates drop id, force

save ncc_nodup, replace


////////////////////////////////////////////////////////////////////////////////
//                               Question 3 (iii)     
//
//-(iii) From the skeleton of the cohort, find the weights for all individuals 
//       and merge these to the data from (ii) using the unique ID 
//       (Note: the weight should be 1 for cases as all cases were selected).
////////////////////////////////////////////////////////////////////////////////

//load skeleton data:
use skeleton_cohort.dta, clear

//obtain riskset info in skeleton cohort:
cohort_km_weights, controls(2) time(t) failure(event) id(id)
keep if event==1
drop weights
save risksets_info, replace

//obtain weights by merging NCC (without duplicates) and risk sets info:
use ncc_nodup, clear
sampled_km_weights using risksets_info, controls(2) id(id) time(t) failure(event) riskset(riskset) numfail(numfail)	

save weights, replace
	

////////////////////////////////////////////////////////////////////////////////
//                               Question 3 (iv)     
//
//-(iv) Sketch the weighted Kaplan-Meier curve and compare to the Kaplan-Meier 
//      curve from the skeleton cohort.
////////////////////////////////////////////////////////////////////////////////

//weighted Kaplan-Meier curve:
use weights, clear
stset t [pw=weights], failure(event)
sts graph, ylabel(0.92(.01)1)
graph export auto1.png, width(500) replace

//Kaplan-Meier curve from the skeleton cohort:
use skeleton_cohort, clear
stset t, failure(event)
sts graph, ylabel(0.92(.01)1)
graph export auto2.png, width(500) replace


////////////////////////////////////////////////////////////////////////////////
//                               Question 3 (v)     
//
//-(v) Conduct a weighted Cox regression of the nested case-control data and 
//     verify that the estimates are close to the estimates from the full 
//     cohort shown in (i) above.
////////////////////////////////////////////////////////////////////////////////

use weights, clear
stset t [pw=weights], failure(event) 
stcox centered_Chol centered_HDL centered_SBP Gender Treat Smoke i.agecat, vce(robust) 
